-- ElvUI Titles Datatext
-- By: Lockslap, US - Bleeding Hollow
local E, L, V, P, G = unpack(ElvUI)
local DT = E:GetModule('DataTexts')
local LibQTip = LibStub("LibQTip-1.0")

local format	= string.format
local sub		= string.sub
local len		= string.len
local find		= string.find
local join		= string.join
local sort		= table.sort

local CHECKMARK		= "|TInterface\\Buttons\\UI-CheckBox-Check:0|t"
local displayString = ""
local noTitles		= ""
local titles		= {}
local lastPanel	
local Tip

local function GetTitleFormat(data)
	if data == -1 then return L["|cff71d5ffNone|r"] end
	local title, formatTitle, replace, name = GetTitleName(data), "", "", ""
	if title:sub(1, 1) == " " then
		if title:find("Jenkins") == nil and title:sub(2, 3) ~= "of" and title:sub(2, 4) ~= "the" then
			replace = "%s,"
		else
			replace = "%s"
		end
		formatTitle = replace .. "|cff71d5ff" .. title .. "|r"
	else
		formatTitle = "|cff71d5ff" .. title .. "|r%s"
	end
	
	if not titles_db.useName then
		name = "|cffa6c939<name>|r"
	else
		local _, classFile = UnitClass("player")
		local player, nameRGB = UnitName("player"), RAID_CLASS_COLORS[classFile]
		local nameHex = ("%02x%02x%02x"):format(nameRGB.r * 255, nameRGB.g * 255, nameRGB.b * 255)
		name = ("|cff%s%s|r"):format(nameHex, player)
	end
	
	return formatTitle:format(name)
end

local function UpdateTitles()
	titles = {}
	for i = 1, GetNumTitles() do
		if IsTitleKnown(i) ~= 0 then
			local title = GetTitleName(i)
			local current = GetCurrentTitle() == i and true or false
			titles[#titles + 1] = {
				id			= i,
				name		= title:sub(1, 1) == " " and title:sub(2) or title:sub(1, title:len() - 1),
				formatName	= GetTitleFormat(i),
				current		= current,
			}
		end
	end
	-- in case they dont wish to use a title
	titles[#titles + 1] = {
		id			= -1,
		name		= L["None"],
		formatName	= L["|cff71d5ffNone|r"],
		current		= nil,
	}
	sort(titles, function(a, b) return a["name"] < b["name"] end)
end

StaticPopupDialogs["CONFIRM_CHANGE_TITLE"] = {
	text			= L["Change title to \"%s\"?"],
	button1			= L["Yes"],
	button2			= L["No"],
	OnAccept		= function(self, data)
		local tName = data ~= -1 and GetTitleFormat(data) or L["|cff71d5ffNone|r"]
		SetCurrentTitle(data)
		if data ~= -1 then
			DEFAULT_CHAT_FRAME:AddMessage(format(L["Title changed to \"%s\"."], tName), 1.0, 1.0, 0)
		else
			DEFAULT_CHAT_FRAME:AddMessage(L["You have elected not to use a title."], 1.0, 1.0, 0)
		end
	end,
	timeout			= 10,
	whileDead		= true,
	hideOnEscape	= true
}

local function OnEvent(self, event, ...)
	UpdateTitles()
	if #titles == 0 then
		self.text:SetFormattedText(noTitles, L["No Titles"])
	else
		self.text:SetFormattedText(displayString, L["Titles"], #titles)
	end
end

local function TitleClick(frame, info, button)
	local dialog = StaticPopup_Show("CONFIRM_CHANGE_TITLE", GetTitleFormat(info))
	if dialog then dialog.data = info end
end

local function OnEnter(self)
	if InCombatLockdown() then return end
	UpdateTitles()
	
	if LibQTip:IsAcquired("TitlesDatatext") then
		Tip:Clear()
	else
		Tip = LibQTip:Acquire("TitlesDatatext", 2, "CENTER", "LEFT")
		Tip:SmartAnchorTo(self)
		Tip:SetAutoHideDelay(0.1, self)
		Tip:SetScript("OnShow", function(ttskinself) ttskinself:SetTemplate("Transparent") end)
	end
	
	line = Tip:AddHeader()
	Tip:SetCell(line, 1, "Titles", "LEFT", 2)
	Tip:AddSeparator()
	
	if #titles == 0 then
		line = Tip:AddLine()
		Tip:SetCell(line, 2, "|cffff0000No Titles Found!|r", "CENTER")
	else
		for _, title in pairs(titles) do
			line = Tip:AddLine()
			if title.current == true then
				Tip:SetCell(line, 1, CHECKMARK, "CENTER")
			end
			
			Tip:SetCell(line, 2, title.formatName, "LEFT")
			Tip:SetLineScript(line, "OnMouseUp", TitleClick, tonumber(title.id))
		end
	end
	
	Tip:UpdateScrolling()
	Tip:Show()
end

local function ValueColorUpdate(hex, r, g, b)
	displayString = join("", "|cffffffff%s:|r", " ", hex, "%d|r")
	noTitles = join("", hex, "%s|r")
	if lastPanel ~= nil then OnEvent(lastPanel) end
end
E["valueColorUpdateFuncs"][ValueColorUpdate] = true

titles_db = titles_db or {
	["useName"]	= true,
}

E.Options.args.datatexts.args.useName = {
	type	= "toggle",
	order	= 4,
	name	= L["Name in Titles"],
	desc	= L["Use your character's class color and name in the tooltip."],
	get		= function(info) return titles_db[info[#info]] end,
	set		= function(info, value) titles_db[info[#info]] = value; DT:LoadDataTexts() end,
}

DT:RegisterDatatext("Titles", {"PLAYER_ENTERING_WORLD", "KNOWN_TITLES_UPDATE"}, OnEvent, nil, nil, OnEnter)